# -*- coding: utf-8 -*-
from __future__ import print_function

from owmeta_core.dataobject import DatatypeProperty, ObjectProperty, Alias

from .muscle import Muscle
from .cell import Cell
from .biology import BiologyType
from .network import Network
from .worm_common import WORM_RDF_TYPE


class Worm(BiologyType):
    """ A representation of the whole worm """

    class_context = BiologyType.class_context

    rdf_type = WORM_RDF_TYPE

    scientific_name = DatatypeProperty()
    ''' Scientific name for the organism '''

    name = Alias(scientific_name)
    ''' Alias to `scientific_name` '''

    muscle = ObjectProperty(value_type=Muscle, multiple=True)
    ''' A type of muscle which is in the worm '''

    cell = ObjectProperty(value_type=Cell)
    ''' A type of cell in the worm '''

    neuron_network = ObjectProperty(value_type=Network, inverse_of=(Network, 'worm'))
    ''' The neuron network of the worm '''

    def __init__(self, scientific_name=None, **kwargs):
        super(Worm, self).__init__(**kwargs)

        if scientific_name:
            self.scientific_name(scientific_name)
        else:
            self.scientific_name("C. elegans")

    def get_neuron_network(self):
        """
        Return the neuron network of the worm.

        Example::

            # Grabs the representation of the neuronal network
            >>> net = P.Worm().get_neuron_network()

            # Grab a specific neuron
            >>> aval = net.aneuron('AVAL')

            >>> aval.type()
            set([u'interneuron'])

            #show how many connections go out of AVAL
            >>> aval.connection.count('pre')
            77

        :returns: An object to work with the network of the worm
        :rtype: owmeta.Network
        """
        return self.neuron_network()

    def muscles(self):
        """
        Get all Muscle objects attached to the Worm.

        Example::

            >>> muscles = P.Worm().muscles()
            >>> len(muscles)
            96

        :returns: A set of all muscles
        :rtype: :py:class:`set`
         """
        return set(x for x in self._muscles_helper())

    def _muscles_helper(self):
        for x in self.muscle.get():
            yield x

    def get_semantic_net(self):
        """
         Get the underlying semantic network as an RDFLib Graph

        :returns: A semantic network containing information about the worm
        :rtype: rdflib.ConjunctiveGraph
         """

        return self.rdf

    def defined_augment(self):
        ''' True if the name is defined '''
        return self.name.has_defined_value()

    def identifier_augment(self, *args, **kwargs):
        ''' Result is derived from the name property '''
        return self.make_identifier(self.name.defined_values[0])



import neuroml as N
from owmeta_core.data import DataUser
from .neuron import Neuron


class NeuroML(DataUser):
    @classmethod
    def generate(cls, o, t=2):
        """
        Get a NeuroML object that represents the given object. The ``type``
        determines what content is included in the NeuroML object:

        :param o: The object to generate neuroml from
        :param t: The what kind of content should be included in the document
                    - 0=full morphology+biophysics
                    - 1=cell body only+biophysics
                    - 2=full morphology only
        :returns: A NeuroML object that represents the given object.
        :rtype: NeuroMLDocument
        """
        if isinstance(o, Neuron):
            # read in the morphology data
            d = N.NeuroMLDocument(id=o.name())
            c = N.Cell(id=o.name())
            c.morphology = o.morphology()
            d.cells.append(c)
            return d
        else:
            raise "Not a valid object for conversion to neuroml"

    @classmethod
    def write(cls, o, n):
        """
        Write the given neuroml document object out to a file
        :param o: The NeuroMLDocument to write
        :param n: The name of the file to write to
        """
        N.writers.NeuroMLWriter.write(o, n)

    @classmethod
    def validate(cls, o):
        pass



from rdflib.term import URIRef

from owmeta_core.dataobject import DatatypeProperty

from . import SCI_CTX
from .document import BaseDocument


class Website(BaseDocument):
    """ A representation of a website """

    class_context = SCI_CTX

    url = DatatypeProperty()
    ''' A URL for the website '''

    title = DatatypeProperty()
    ''' The official name for the website '''

    def __init__(self, title=None, **kwargs):
        super(Website, self).__init__(rdfs_comment=title, title=title, **kwargs)

    def defined_augment(self):
        return self.url.has_defined_value()

    def identifier_augment(self):
        return URIRef(self.url.defined_values[0].identifier)



import traceback

from rdflib.namespace import Namespace

from owmeta_core.dataobject import ObjectProperty
from owmeta_core.datasource import GenericTranslation
from owmeta_core.data_trans.csv_ds import CSVDataSource, CSVDataTranslator

from .. import CONTEXT
from ..network import Network
from ..worm import Worm
from ..website import Website
from ..evidence import Evidence

from .data_with_evidence_ds import DataWithEvidenceDataSource
from .common_data import DTMixin, DSMixin


class WormAtlasCellListDataSource(DSMixin, CSVDataSource):
    class_context = CONTEXT

    csv_header = ['Cell', 'Lineage Name', 'Description']
    csv_field_delimiter = '\t'


class WormAtlasCellListDataTranslation(GenericTranslation):
    class_context = CONTEXT
    neurons_source = ObjectProperty()

    def defined_augment(self):
        return super(WormAtlasCellListDataTranslation, self).defined_augment() and \
                self.neurons_source.has_defined_value()

    def identifier_augment(self):
        return self.make_identifier(super(WormAtlasCellListDataTranslation, self).identifier_augment().n3() +
                                    self.neurons_source.onedef().identifier.n3())


class WormAtlasCellListDataTranslator(DTMixin, CSVDataTranslator):
    class_context = CONTEXT

    input_type = (WormAtlasCellListDataSource, DataWithEvidenceDataSource)
    output_type = DataWithEvidenceDataSource
    translation_type = WormAtlasCellListDataTranslation

    def make_translation(self, sources):
        res = super(WormAtlasCellListDataTranslator, self).make_translation()
        res.source(sources[0])
        res.neurons_source(sources[1])
        return res

    def translate(self, data_source, neurons_source):
        res = self.make_new_output(sources=(data_source, neurons_source))
        net_q = neurons_source.data_context.stored(Network).query()
        net = next(net_q.load(), net_q)

        # TODO: Improve this evidence marker
        doc = res.evidence_context(Website)(url="http://www.wormatlas.org/celllist.htm")
        ev = res.evidence_context(Evidence)(reference=doc)
        doc_ctx = res.data_context_for(document=doc)
        ev.supports(doc_ctx.rdf_object)
        w = doc_ctx(Worm)()

        with self.make_reader(data_source, skipinitialspace=True, skipheader=True) as csvreader:
            cell_name_counters = dict()
            data = dict()
            for j in csvreader:
                name = j[0]
                lineageName = j[1]
                desc = j[2]

                # XXX: These renaming choices are arbitrary; may be inappropriate
                if name == "DB1/3":
                    name = "DB1"
                elif name == "DB3/1":
                    name = "DB3"
                elif name == "AVFL/R":
                    if lineageName[0] == "W":
                        name = "AVFL"
                    elif lineageName[0] == "P":
                        name = "AVFR"

                if name in cell_name_counters:
                    basename = name
                    while name in cell_name_counters:
                        cell_name_counters[basename] += 1
                        name = basename + "(" + str(cell_name_counters[basename]) + ")"
                else:
                    cell_name_counters[name] = 0

                data[name] = {"lineageName": lineageName, "desc": desc}

        for n in net.neurons():
            # Get the name of the neuron in its original context
            name = n.name.one()
            cell_data = data[str(name)]
            # Make statements in the result context
            nn = doc_ctx(n)
            nn.lineageName(cell_data['lineageName'])
            nn.description(cell_data['desc'])
            w.cell(nn)

        # TODO: Add data for other cells here. Requires relating named
        # muscle cells to their counterparts in the cell list (e.g. mu_bod(#))

        print("uploaded lineage and descriptions")
        return res



import bibtexparser

from .evidence import Evidence
from .bibtex_customizations import customizations


def bibtex_to_document(bibtex_entry, context=None):
    """ Takes a single BibTeX entry and translates it into a `.Document` object """
    from owmeta.document import Document

    res = Document.contextualize(context)()
    update_document_with_bibtex(res, bibtex_entry)
    return res


def update_document_with_bibtex(document, bibtex_entry):
    document.key = bibtex_entry['ID']
    for ath in bibtex_entry.get('author', tuple()):
        document.author(ath)

    fields = ['title',
              'year',
              'author',
              'doi',
              ('link', 'uri')]
    for x in fields:
        if isinstance(x, tuple):
            key, prop = x
        else:
            prop = x
            key = x
        for m in bibtex_entry.get(key, ()):
            getattr(document, prop)(m)


def make_default_bibtex_parser():
    parser = bibtexparser.bparser.BibTexParser(common_strings=True)
    parser.customization = customizations
    return parser


def loads(bibtex_string):
    '''
    Load BibTeX records from a string

    Parameters
    ----------
    bibtex_string : str
        Text of one or more BibTex records

    Returns
    -------
    bibtexparser.bibdatabase.BibDatabase
        Records represented in the string
    '''
    parser = make_default_bibtex_parser()
    return bibtexparser.loads(bibtex_string, parser=parser)


def load(bibtex_file):
    '''
    Load BibTeX records from a file

    Parameters
    ----------
    bibtex_file : :term:`file object`
        File containing one or more BibTex records

    Returns
    -------
    bibtexparser.bibdatabase.BibDatabase
        Records represented in the string
    '''
    parser = make_default_bibtex_parser()
    return bibtexparser.load(bibtex_file, parser=parser)


def load_from_file_named(file_name):
    '''
    Loads from a file with the given name

    Parameters
    ----------
    file_name : str
        Name of the bibtex file to open

    Returns
    -------
    bibtexparser.bibdatabase.BibDatabase
        Records from the named file
    '''
    with open(file_name) as bibtex_file:
        return load(bibtex_file)


def parse_bibtex_into_documents(file_name, context=None):
    '''
    Parses BibTeX records into a dictionary of `.Document` instances

    Parameters
    ----------
    bibtex_file : :term:`file object`
        File containing one or more BibTeX records

    Returns
    -------
    dict
        `.Document` instances from the records in the file
    '''
    res = dict()
    bib_database = load_from_file_named(file_name)
    for entry in bib_database.entries:
        entry_id = entry['ID']
        res[entry_id] = bibtex_to_document(entry, context)

    return res


def parse_bibtex_into_evidence(file_name, context=None):
    return {k: Evidence.contextualize(context)(reference=v, supports=v.contextualize(context).as_context.rdf_object)
            for k, v
            in parse_bibtex_into_documents(file_name, context).items()}



from __future__ import print_function

from owmeta_core.dataobject import DatatypeProperty, ObjectProperty, This

from .channel import Channel
from .biology import BiologyType
from .cell_common import CELL_RDF_TYPE


class Cell(BiologyType):
    """
    A biological cell.

    All cells with the same `name` are considered to be the same object.

    Parameters
    -----------
    name : str
        The name of the cell
    lineageName : str
        The lineageName of the cell

    Examples
    --------
    >>> from owmeta_core.quantity import Quantity
    >>> c = Cell(lineageName="AB plapaaaap",
    ...     divisionVolume=Quantity("600","(um)^3"))
    """

    class_context = BiologyType.class_context

    rdf_type = CELL_RDF_TYPE

    divisionVolume = DatatypeProperty()
    ''' The volume of the cell at division '''

    name = DatatypeProperty()
    ''' The 'adult' name of the cell typically used by biologists when discussing C. elegans '''

    wormbaseID = DatatypeProperty()

    description = DatatypeProperty()
    ''' A description of the cell '''

    channel = ObjectProperty(value_type=Channel,
                             multiple=True,
                             inverse_of=(Channel, 'appearsIn'))

    lineageName = DatatypeProperty()
    ''' The lineageName of the cell '''

    synonym = DatatypeProperty(multiple=True)

    daughterOf = ObjectProperty(value_type=This,
                                inverse_of=(This, 'parentOf'))

    parentOf = ObjectProperty(value_type=This, multiple=True)

    key_property = 'name'

    direct_key = True

    def __init__(self, name=None, lineageName=None, **kwargs):
        # NOTE: We name the `name` and `lineageName` as positional parameters for
        # convenience
        super(Cell, self).__init__(name=name, lineageName=lineageName, **kwargs)

    def blast(self):
        """
        Return the blast name.

        Example::

            >>> c = Cell(name="ADAL", lineageName='AB ')
            >>> c.blast()
            'AB'

        Note that this isn't a `~dataobject_property.Property`. It returns the blast cell
        part of a `lineageName` value.
        """
        import re
        try:
            ln = self.lineageName()
            x = re.split("[. ]", ln)
            return x[0]
        except Exception:
            return ""

    def __str__(self):
        if self.name.has_defined_value():
            return str(self.name.defined_values[0].idl)
        else:
            return super(Cell, self).__str__()



from owmeta_core.context import Context
from owmeta_core.datasource import Informational, DataSource
from owmeta_core.data_trans.context_datasource import (VariableIdentifierContext,
        VariableIdentifierContextDataObject)

from .. import CONTEXT, SCI_CTX

from .common_data import DSMixin


class DataWithEvidenceDataSource(DSMixin, DataSource):
    '''
    A data source that has an "evidence context" containing statements which support those
    in its "data context". The data source also has a combined context which imports both
    the data and evidence contexts. The data and evidence contexts have identifiers based
    on the data source's identifier and the combined context has the same identifier as
    the data source.
    '''

    class_context = SCI_CTX

    evidence_context_property = Informational(display_name='Evidence context',
                                              property_name='evidence_context',
                                              property_type='ObjectProperty',
                                              multiple=False,
                                              value_type=VariableIdentifierContextDataObject,
                                              description='The context in which evidence'
                                                          ' for the "Data context" is defined')

    data_context_property = Informational(display_name='Data context',
                                          property_name='data_context',
                                          property_type='ObjectProperty',
                                          multiple=False,
                                          value_type=VariableIdentifierContextDataObject,
                                          description='The context in which primary data'
                                                      ' for this data source is defined')

    combined_context_property = Informational(display_name='Combined context',
                                              property_name='combined_context',
                                              property_type='ObjectProperty',
                                              multiple=False,
                                              value_type=VariableIdentifierContextDataObject,
                                              description='Context importing both the data and evidence contexts')

    def __init__(self, *args, **kwargs):
        super(DataWithEvidenceDataSource, self).__init__(*args, **kwargs)

        self.__ad_hoc_contexts = dict()

        self.data_context = _DataContext.contextualize(self.context)(maker=self,
                                                                     imported=(CONTEXT,))

        self.evidence_context = _EvidenceContext.contextualize(self.context)(maker=self,
                                                                             imported=(CONTEXT,))

        self.combined_context = _CombinedContext.contextualize(self.context)(maker=self,
                                                                             imported=(self.data_context,
                                                                                       self.evidence_context))
        if not type(self).query_mode:
            if not self.data_context_property.has_defined_value():
                self.data_context_property(self.data_context.rdf_object)

            if not self.evidence_context_property.has_defined_value():
                self.evidence_context_property(self.evidence_context.rdf_object)

            if not self.combined_context_property.has_defined_value():
                self.combined_context_property(self.combined_context.rdf_object)

    def data_context_for(self, **kwargs):
        ctx = self.context_for(**kwargs)
        self.data_context.add_import(ctx)
        return ctx

    def context_for(self, ident=None, **kwargs):
        key = "&".join(k + "=" + getattr(kwargs[k], 'identifier', str(kwargs[k]))
                       for k in sorted(kwargs.keys()))
        res = self.__ad_hoc_contexts.get(key)
        if res is None:
            if ident:
                ctxid = ident
            else:
                ctxid = self.identifier + '/context_for?' + key
            self.__ad_hoc_contexts[key] = Context.contextualize(self.context)(ident=ctxid)
            res = self.__ad_hoc_contexts[key]
        return res

    def after_transform(self):
        super(DataWithEvidenceDataSource, self).after_transform()
        for ctx in self.__ad_hoc_contexts.values():
            ctx.save()

        self.evidence_context.save()
        self.data_context.save()
        self.combined_context.save()

        for ctx in self.__ad_hoc_contexts.values():
            ctx.save_imports(transitive=False)

        self.evidence_context.save_imports(transitive=False)
        self.data_context.save_imports(transitive=False)
        self.combined_context.save_imports(transitive=False)


class _CombinedContext(VariableIdentifierContext):
    def identifier_helper(self):
        if self.maker.defined:
            return self.maker.identifier
        else:
            return None


class _EvidenceContext(VariableIdentifierContext):
    def identifier_helper(self):
        if self.maker.defined:
            return self.maker.identifier + '-evidence'
        else:
            return None


class _DataContext(VariableIdentifierContext):
    def identifier_helper(self):
        if self.maker.defined:
            return self.maker.identifier + '-data'
        else:
            return None



# -*- coding: utf-8 -*-

"""
.. _owm_module:

owmeta
======

OpenWorm Unified Data Abstract Layer.

An introduction to owmeta can be found in the README on our
`Github page <https://github.com/openworm/owmeta>`_.
"""

from __future__ import print_function
__version__ = '0.12.4.dev20220619003248'
__author__ = 'OpenWorm.org authors and contributors'

import logging

LOGGER = logging.getLogger(__name__)
LOGGER.addHandler(logging.NullHandler())

BASE_SCHEMA_URL = 'http://schema.openworm.org/2020/07/sci'
BASE_BIO_SCHEMA_URL = 'http://schema.openworm.org/2020/07/sci/bio'
BASE_DATA_URL = 'http://data.openworm.org/sci'
BASE_BIO_DATA_URL = 'http://data.openworm.org/sci/bio'

from owmeta_core import BASE_CONTEXT
from owmeta_core.context import Context, ClassContext

DEF_CTX = Context()

SCI_CTX = ClassContext(imported=(BASE_CONTEXT,),
                      ident=BASE_SCHEMA_URL,
                      base_namespace=BASE_SCHEMA_URL + '#')

CONTEXT = ClassContext(imported=(SCI_CTX,),
                  ident=BASE_BIO_SCHEMA_URL,
                  base_namespace=BASE_BIO_SCHEMA_URL + '#')



from owmeta_core.datasource import Informational, DataTranslator, DataSource
from owmeta_core.data_trans.local_file_ds import LocalFileDataSource
from owmeta_core.data_trans.context_datasource import VariableIdentifierContext

from .. import CONTEXT, SCI_CTX
from ..bibtex import parse_bibtex_into_evidence

from .common_data import DSMixin, DTMixin


class EvidenceDataSource(DSMixin, DataSource):
    class_context = SCI_CTX

    context_property = Informational(display_name='Context',
                                     property_name='evidence_context',
                                     property_type='ObjectProperty',
                                     description='The context')

    def __init__(self, *args, **kwargs):
        super(EvidenceDataSource, self).__init__(*args, **kwargs)
        self.context = _EvidenceContext.contextualize(self.context)(maker=self,
                                                                    imported=(CONTEXT,))
        self.context_property(self.evidence_context.rdf_object)

    def after_transform(self):
        super(EvidenceDataSource, self).after_transform()
        saved_contexts = set([])
        self.data_context.save_context(inline_imports=True, saved_contexts=saved_contexts)
        self.context.save_context(inline_imports=True, saved_contexts=saved_contexts)
        self.context.save_imports()


class _EvidenceContext(VariableIdentifierContext):
    def identifier_helper(self):
        return self.maker.identifier + '-evidence'


class BibTexDataSource(DSMixin, LocalFileDataSource):
    class_context = SCI_CTX

    def __init__(self, bibtex_file_name, **kwargs):
        super(BibTexDataSource, self).__init__(**kwargs)
        self.bibtex_file_name = bibtex_file_name


class BibTexDataTranslator(DTMixin, DataTranslator):
    class_context = SCI_CTX

    input_type = BibTexDataSource
    output_type = EvidenceDataSource

    def translate(data_source):
        evidences = parse_bibtex_into_evidence(data_source.bibtex_file_name)
        return evidences.values()



from .muscle import Muscle
from .neuron import Neuron
from .data_trans.neuron_data import NeuronCSVDataSource
from .data_trans.wormatlas import WormAtlasCellListDataSource
from .data_trans.wormbase import (WormBaseCSVDataSource, WormbaseIonChannelCSVDataSource,
                                  WormbaseTextMatchCSVDataSource)
from .data_trans.connections import ConnectomeCSVDataSource


def owm_data(ns):
    '''
    Sources based on objects external to owmeta (e.g., files, websites)
    '''
    ctx = ns.context
    ctx.add_import(ConnectomeCSVDataSource.definition_context)

    ctx(ConnectomeCSVDataSource)(
            key='connectome',
            csv_file_name="connectome.csv")
    ctx(NeuronCSVDataSource)(
            key='bently_expression',
            csv_file_name="expression_data/Bentley_et_al_2016_expression.csv")
    ctx(NeuronCSVDataSource)(
            key='neurons',
            csv_file_name="Modified celegans db dump.csv",
            bibtex_files=["bibtex_files/altun2009.bib", "bibtex_files/WormAtlas.bib"])
    ctx(WormAtlasCellListDataSource)(
            key='cells',
            csv_file_name="C. elegans Cell List - WormAtlas.tsv",
            description="CSV converted from this Google Spreadsheet: "
            "https://docs.google.com/spreadsheets/d/"
            "1Jc9pOJAce8DdcgkTgkUXafhsBQdrer2Y47zrHsxlqWg/edit")
    ctx(WormbaseIonChannelCSVDataSource)(
            key='ion_channels',
            csv_file_name="ion_channel.csv",
            description="C. elegans ion channel data from WormBase. Contains"
            " WormBase IDs (WBIDs) for genes that code the ion channels and for"
            " the expression of those genes in C. elegans (e.g.,"
            " \"Chronogram479\", \"Expr6468\")")
    ctx(WormbaseTextMatchCSVDataSource)(
            key='muscle_ion_channels',
            csv_file_name="Ion channels - Ion Channel To Body Muscle.tsv",
            cell_type=Muscle.rdf_type,
            initial_cell_column=6)
    ctx(WormbaseTextMatchCSVDataSource)(
            key='neuron_ion_channels',
            csv_file_name="Ion channels - Ion Channel To Neuron.tsv",
            cell_type=Neuron.rdf_type,
            initial_cell_column=101)
    ctx(ConnectomeCSVDataSource)(
            key="emmons",
            csv_file_name="herm_full_edgelist.csv")
    ctx(WormBaseCSVDataSource)(
            key="wormbase_celegans_cells",
            csv_file_name="C. elegans Cell List - WormBase.csv",
            description="CSV converted from this Google Spreadsheet: "
            "https://docs.google.com/spreadsheets/d/"
            "1NDx9LRF_B2phR5w4HlEtxJzxx1ZIPT2gA0ZmNmozjos/edit#gid=1")



from owmeta_core.cli_common import METHOD_NAMED_ARG

CLI_HINTS = {
    'owmeta.command.OWMEvidence': {
        'get': {
            (METHOD_NAMED_ARG, 'identifier'): {
                'names': ['identifier'],
            },
        },
    },
    'owmeta.commands.biology.CellCmd': {
        'show': {
            (METHOD_NAMED_ARG, 'cell_name_or_id'): {
                'names': ['cell_name_or_id'],
            },
        },
    },
}



from owmeta_core.dataobject import DatatypeProperty, ObjectProperty, Alias

from .cell import Cell
from .neuron import Neuron


class Muscle(Cell):
    """
    A single muscle cell.

    See what neurons innervate a muscle:

    Example::

        >>> mdr21 = Muscle('MDR21')
        >>> innervates_mdr21 = mdr21.innervatedBy()
        >>> len(innervates_mdr21)
        4
    """

    class_context = Cell.class_context

    innervatedBy = ObjectProperty(value_type=Neuron, multiple=True)
    ''' Neurons synapsing with this muscle '''

    neurons = Alias(innervatedBy)
    ''' Alias to `innervatedBy` '''

    receptors = DatatypeProperty(multiple=True)
    ''' Receptor types expressed by this type of muscle '''

    receptor = Alias(receptors)
    ''' Alias to `receptors` '''


class BodyWallMuscle(Muscle):
    '''
    A somatic muscle cell that lies close under the skin and basal lamina of C. elegans
    and allows the worm to move
    '''
    class_context = Cell.class_context



'''
Data translators

Some `DataSource` and `DataTranslator` types. Some deal with generic file types (e.g., comma-separated values) while
others are specific to the format of a kind of file housed in owmeta.

'''



from rdflib.namespace import Namespace

from owmeta_core.datasource import Informational
from owmeta_core.data_trans.csv_ds import CSVDataSource, CSVDataTranslator

from .. import CONTEXT
from ..neuron import Neuron
from ..network import Network
from ..worm import Worm
from ..document import Document
from ..evidence import Evidence
from ..bibtex import parse_bibtex_into_documents

from .common_data import DSMixin, DTMixin
from .data_with_evidence_ds import DataWithEvidenceDataSource


class NeuronCSVDataSource(DSMixin, CSVDataSource):
    class_context = CONTEXT

    bibtex_files = Informational(display_name='BibTeX files',
                                 description='List of BibTeX files that are referenced in the csv file by entry ID')


class NeuronCSVDataTranslator(DTMixin, CSVDataTranslator):
    class_context = CONTEXT

    input_type = NeuronCSVDataSource
    output_type = DataWithEvidenceDataSource

    def __init__(self, *args, **kwargs):
        super(NeuronCSVDataTranslator, self).__init__(*args, **kwargs)
        self.__document_contexts = dict()

    def translate(self, data_source):
        res = self.make_new_output((data_source,))

        documents = dict()
        bibtex_files = data_source.bibtex_files.onedef()
        if bibtex_files is not None:
            for bib in bibtex_files:
                documents.update(parse_bibtex_into_documents(bib, res.evidence_context))
        worm = res.data_context(Worm)()
        network = res.data_context(Network)(worm=worm)
        with self.make_reader(data_source, skipheader=True) as reader:
            for row in reader:
                neuron_name, relation, data, evidence, documentURL = row
                relation = relation.lower()

                docs = []
                doc = documents.get(evidence, None)
                if doc is not None:
                    docs.append(doc)

                if len(documentURL) > 0:
                    doc1 = documents.get(documentURL, res.evidence_context(Document)(uri=documentURL))
                    documents[documentURL] = doc1
                    docs.append(doc1)

                if relation in ('neurotransmitter', 'innexin', 'neuropeptide', 'receptor'):
                    for d in docs:
                        neu = res.data_context_for(document=d)(Neuron)(neuron_name)
                        network.neuron(neu)
                        getattr(neu, relation)(data)
                elif relation == 'type':
                    _data = data.lower()
                    # type data aren't normalized so we check for strings within the _data string
                    types = [x for x in ('sensory', 'interneuron', 'motor', 'unknown') if x in _data]

                    for t in types:
                        for d in docs:
                            neu = res.data_context_for(document=d)(Neuron)(neuron_name)
                            network.neuron(neu)
                            neu.type(t)
        for d in documents.values():
            contextualized_doc_ctx = res.evidence_context(res.data_context_for(document=d))
            res.evidence_context(Evidence)(reference=d, supports=contextualized_doc_ctx.rdf_object)
            res.data_context.add_import(contextualized_doc_ctx)
        return res



# -*- coding: utf-8 -*-
#

from setuptools import setup


long_description = """
owmeta
======

A unified, simple data access library in Python for data, facts, and models of
*C. elegans* anatomy for the `OpenWorm project <http://www.openworm.org>`_

What does it do?
----------------

Enables a simple Python API for asking various questions about the cells of the
*C. elegans*, enabling the sharing of data about *C. elegans* for the purpose
of building a `data-to-model pipeline <http://docs.openworm.org/en/latest/projects>`_
for the OpenWorm project. In addition, it is a repository for various iterations
of inferred / predicted data about *C. elegans*. Uncontroversial facts and
inferred information are distinguished through the use of explicit Evidence
references.
"""


for line in open('owmeta/__init__.py'):
    if line.startswith("__version__"):
        version = line.split("=")[1].strip()[1:-1]


setup(
    name='owmeta',
    zip_safe=False,
    install_requires=[
        'owmeta-core>=0.14.0.dev0',
        'bibtexparser~=1.1.0',
        'libneuroml',
        'rdflib>=4.1.2',
        'six~=1.10',
        'requests',
    ],
    version=version,
    packages=['owmeta',
              'owmeta.data_trans',
              'owmeta.commands'],
    author='OpenWorm.org authors and contributors',
    author_email='info@openworm.org',
    description='A Python library for working with OpenWorm data and models',
    long_description=long_description,
    license='MIT',
    url='https://owmeta.readthedocs.io/en/latest/',
    download_url='https://github.com/openworm/owmeta/archive/master.zip',
    entry_points={
        'owmeta_core.commands': [
            'evidence = owmeta.command:OWMEvidence',
            'cell = owmeta.commands.biology:CellCmd',
        ],
        'owmeta_core.cli_hints': 'hints = owmeta.cli_hints:CLI_HINTS',
    },
    classifiers=[
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Scientific/Engineering'
    ]
)



import six
from .experiment import Experiment

from owmeta_core.dataobject import DataObject, DatatypeProperty, ObjectProperty, CPThunk

from . import CONTEXT
from .channel_common import CHANNEL_RDF_TYPE
from .neuroml import NeuroMLProperty


class PatchClampExperiment(Experiment):
    """
    Store experimental conditions for a patch clamp experiment.
    """

    class_context = CONTEXT

    Ca_concentration = DatatypeProperty()
    '''Calcium concentration'''

    Cl_concentration = DatatypeProperty()
    '''Chlorine concentration'''

    blockers = DatatypeProperty()
    '''Channel blockers used for this experiment'''

    cell = DatatypeProperty()
    '''The cell this experiment was performed on'''

    cell_age = DatatypeProperty()
    '''Age of the cell'''

    delta_t = DatatypeProperty()

    duration = DatatypeProperty()

    end_time = DatatypeProperty()

    extra_solution = DatatypeProperty()

    initial_voltage = DatatypeProperty()
    '''Starting voltage of the patch clamp'''

    ion_channel = DatatypeProperty()
    '''The ion channel being clamped'''

    membrane_capacitance = DatatypeProperty()
    '''Initial membrane capacitance'''

    mutants = DatatypeProperty()
    '''Type(s) of mutants being used in this experiment'''

    patch_type = DatatypeProperty()
    '''Type of patch clamp being used ('voltage' or 'current')'''

    pipette_solution = DatatypeProperty()
    '''Type of solution in the pipette'''

    protocol_end = DatatypeProperty()

    protocol_start = DatatypeProperty()

    protocol_step = DatatypeProperty()

    start_time = DatatypeProperty()

    temperature = DatatypeProperty()

    type = DatatypeProperty()


class ChannelModelType:
    patchClamp = "Patch clamp experiment"
    homologyEstimate = "Estimation based on homology"


class ChannelModel(DataObject):
    """
    A model for an ion channel.

    There may be multiple models for a single channel.

    Example usage::

        >>> from owmeta_core.quantity import Quantity

        # Create a ChannelModel
        >>> cm = PatchClampChannelModel(key='ca_boyle',
        ...     gating='voltage',
        ...     ion='Ca',
        ...     conductance=Quantity.parse('10pS'))

    """
    class_context = CONTEXT

    modelType = DatatypeProperty()
    ''' The type of model employed to describe a channel '''

    ion = DatatypeProperty(multiple=True)
    ''' The type of ion this channel selects for '''

    gating = DatatypeProperty(multiple=True)
    ''' The gating mechanism for this channel ("voltage" or name of ligand(s) ) '''

    conductance = DatatypeProperty()
    ''' The conductance of this ion channel. This is the initial value, and should be entered as a Quantity object. '''

    neuroML = CPThunk(NeuroMLProperty)

    def __init__(self, modelType=None, *args, **kwargs):
        super(ChannelModel, self).__init__(*args, **kwargs)

        #Change modelType value to something from ChannelModelType class on init
        if isinstance(modelType, six.string_types):
            modelType = modelType.lower()
            if modelType in ('homology', ChannelModelType.homologyEstimate):
                self.modelType(ChannelModelType.homologyEstimate)
            elif modelType in ('patch-clamp', ChannelModelType.patchClamp):
                self.modelType(ChannelModelType.patchClamp)


class PatchClampChannelModel(ChannelModel):
    class_context = CONTEXT

    modeled_from = ObjectProperty(value_type=PatchClampExperiment)

    def __init__(self, **kwargs):
        super(PatchClampChannelModel, self).__init__(modelType='patch-clamp',
                                                     **kwargs)


class HomologyChannelModel(ChannelModel):
    class_context = CONTEXT

    homolog = ObjectProperty(value_rdf_type=CHANNEL_RDF_TYPE)

    def __init__(self, **kwargs):
        super(HomologyChannelModel, self).__init__(modelType='homology',
                                                   **kwargs)



from owmeta_core.dataobject import DataObject
import rdflib as R

from . import BASE_BIO_DATA_URL, BASE_BIO_SCHEMA_URL


class BiologyType(DataObject):
    class_context = BASE_BIO_SCHEMA_URL

    base_namespace = R.Namespace(BASE_BIO_SCHEMA_URL + '/')
    base_data_namespace = R.Namespace(BASE_BIO_DATA_URL + '/')



from __future__ import print_function
from __future__ import absolute_import
from __future__ import unicode_literals

import six
from owmeta_core.dataobject import DatatypeProperty, ObjectProperty

from .biology import BiologyType
from .cell import Cell

__all__ = ['Connection']


class SynapseType:
    Chemical = 'send'
    GapJunction = 'gapJunction'


class Termination:
    Neuron = 'neuron'
    Muscle = 'muscle'


class Connection(BiologyType):

    class_context = BiologyType.class_context

    post_cell = ObjectProperty(value_type=Cell)
    ''' The post-synaptic cell '''

    pre_cell = ObjectProperty(value_type=Cell)
    ''' The pre-synaptic cell '''

    number = DatatypeProperty()
    ''' The weight of the connection '''

    synclass = DatatypeProperty()
    ''' The kind of Neurotransmitter (if any) sent between `pre_cell` and `post_cell` '''

    syntype = DatatypeProperty()
    ''' The kind of synaptic connection. 'gapJunction' indicates a gap junction and 'send' a chemical synapse '''

    termination = DatatypeProperty()
    ''' Where the connection terminates. Inferred from type of post_cell at initialization '''

    key_properties = (pre_cell, post_cell, syntype)

    # Arguments are given explicitly here to support positional arguments
    def __init__(self,
                 pre_cell=None,
                 post_cell=None,
                 number=None,
                 syntype=None,
                 synclass=None,
                 termination=None,
                 **kwargs):
        super(Connection, self).__init__(pre_cell=pre_cell,
                                         post_cell=post_cell,
                                         number=number,
                                         syntype=syntype,
                                         synclass=synclass,
                                         **kwargs)

        if isinstance(termination, six.string_types):
            termination = termination.lower()
            if termination in ('neuron', Termination.Neuron):
                self.termination(Termination.Neuron)
            elif termination in ('muscle', Termination.Muscle):
                self.termination(Termination.Muscle)

        if isinstance(syntype, six.string_types):
            syntype = syntype.lower()
            if syntype in ('send', SynapseType.Chemical):
                self.syntype(SynapseType.Chemical)
            elif syntype in ('gapjunction', SynapseType.GapJunction):
                self.syntype(SynapseType.GapJunction)

    def __str__(self):
        nom = []
        props = ('pre_cell', 'post_cell', 'syntype', 'termination', 'number', 'synclass',)
        for p in props:
            if getattr(self, p).has_defined_value():
                nom.append((p, getattr(self, p).defined_values[0]))
        if len(nom) == 0:
            return super(Connection, self).__str__()
        else:
            return 'Connection(' + \
                   ', '.join('{}={}'.format(n[0], n[1]) for n in nom) + \
                   ')'
